@@ -29,7 +29,7 @@ vardomh <- function(Y, H, PSU, w_final,
           stop("'confidence' must be a numeric value in [0,1]")  }
 
   if(!is.null(dataset)) {
-      data <- data.frame(dataset)
+      dataset <- data.frame(dataset)
       aY <- Y
       if (min(Y %in% names(dataset))!=1) stop("'Y' does not exist in 'dataset'!")
       if (min(Y %in% names(dataset))==1) {
@@ -167,11 +167,10 @@ vardomh <- function(Y, H, PSU, w_final,
                 stop("'period' are duplicate column names: ", 
                      paste(names(period)[duplicated(names(period))], collapse = ","))
       if (nrow(period) != n) stop("'period' must be the same length as 'inc'")
-      if(any(is.na(period))) stop("'period' has unknown values")  
+      if(any(is.na(period))) stop("'period' has unknown values")
   } 
   np <- sum(ncol(period))
 
-
   # N_h
   if (!is.null(N_h)) {
       N_h <- data.table(N_h)
@@ -203,13 +202,32 @@ vardomh <- function(Y, H, PSU, w_final,
   
   # Z
   if (!is.null(Z)) {
-    Z <- data.table(Z)
+    Z <- data.table(Z, check.names = T)
     if (nrow(Z) != n) stop("'Z' and 'Y' must be equal row count")
     if (ncol(Z) != m) stop("'Z' and 'Y' must be equal column count")
     if (any(is.na(Z))) stop("'Z' has unknown values")
     if (is.null(names(Z))) stop("'Z' must be colnames")
   }
-      
+ 
+  # periodX
+  if (!is.null(X)) {
+     if(!is.null(periodX)) {
+        periodX <- data.table(periodX)
+        periX <- data.table(unique(periodX))
+        setkeyv(periX, names(periX))
+        peri <- data.table(unique(period))
+        setkeyv(peri, names(peri))
+        if (any(duplicated(names(periodX)))) 
+                    stop("'periodX' are duplicate column names: ", 
+                         paste(names(periodX)[duplicated(names(periodX))], collapse = ","))
+        if (nrow(periodX) != nrow(X)) stop("'periodX' length must be equal with 'X' row count")
+        if (ncol(periodX) != ncol(period)) stop("'periodX' length must be equal with 'period' column count")
+        if (names(periodX) != names(period)) stop("'periodX' must be equal with 'period' names")
+        if (any(is.na(periodX))) stop("'periodX' has unknown values")
+        if (any(peri != periX)) stop("'unique(period)' and 'unique(periodX)' records have different")
+      }
+   }
+
  # X_ID_household
   if (!is.null(X)) {
     X_ID_household <- data.table(X_ID_household)
@@ -217,12 +235,12 @@ vardomh <- function(Y, H, PSU, w_final,
     if (any(is.na(X_ID_household))) stop("'X_ID_household' has unknown values")
 
     IDh <- data.table(unique(ID_household))
-    if (!is.null(period)) {X_ID_household <- data.table(periodX, X_ID_household)
+    if (!is.null(periodX)) {X_ID_household <- data.table(periodX, X_ID_household)
                            IDh <- data.table(unique(data.table(period, ID_household)))}
+
     if (any(duplicated(X_ID_household))) stop("'X_ID_household' have duplicates")
     setkeyv(X_ID_household, names(X_ID_household))
     setkeyv(IDh, names(IDh))
-
     if (!is.null(period)) {
         if (nrow(IDh) != nrow(X_ID_household)) stop("'period' with 'X_ID_household' and 'unique(period, ID_household)' have different row count")
         if (any(IDh != X_ID_household)) stop("'period' with 'X_ID_household' and 'unique(period, ID_household)' records have different")
@@ -238,22 +256,7 @@ vardomh <- function(Y, H, PSU, w_final,
     if (nrow(X) != nrow(X_ID_household)) stop("'X' and 'X_ID_household' have different row count")
   }
 
-  # periodX
-  if (!is.null(X)) {
-     if(!is.null(periodX)) {
-        periodX <- data.table(periodX)
-        periX <- data.table(unique(periodX))
-        setkeyv(periX, names(periX))
-        peri <- data.table(unique(period))
-        setkeyv(peri, names(peri))
-        if (any(duplicated(names(periodX)))) 
-                    stop("'periodX' are duplicate column names: ", 
-                         paste(names(periodX)[duplicated(names(periodX))], collapse = ","))
-        if (nrow(periodX) != nrow(X)) stop("'periodX' length must be equal with 'X' row count")
-        if (any(is.na(periodX))) stop("'periodX' has unknown values")
-        if (any(peri != periX)) stop("'unique(period)' and 'unique(periodX)' records have different")
-      }
-   }
+
 
   # ind_gr
   if (!is.null(X)) {
@@ -273,7 +276,7 @@ vardomh <- function(Y, H, PSU, w_final,
        X2 <- data.table(ind_gr1, X1)
        X1 <- X2[, .N, keyby=names(ind_gr1)][[ncol(ind_gr1)+1]]
        X2 <- X2[,lapply(.SD, function(x) sum(!is.na(x))), keyby=names(ind_gr1), .SDcols=nX1]
-       X2 <- X2[, !(names(X2) %in% names(ind_gr)), with=F]
+       X2 <- X2[, !(names(X2) %in% names(ind_gr1)), with=F]
        if (!all(X2==0 | X1==X2)) stop("X has unknown values")
        ind_gr1 <- nX1 <- X1 <- X2 <- NULL
     }
@@ -306,12 +309,8 @@ vardomh <- function(Y, H, PSU, w_final,
   ### Calculation
       
   # Domains
-  Dom1 <- Dom
-  if (!is.null(period)) {
-     if (!is.null(Dom1)) { Dom1 <- data.table(period, Dom1)
-        } else Dom1 <- period } 
 
-  if (!is.null(Dom1)) Y1 <- domain(Y, Dom1) else Y1 <- Y
+  if (!is.null(Dom)) Y1 <- domain(Y, Dom) else Y1 <- Y
 
   # Design weights
   if (!is.null(X)) {
@@ -326,28 +325,46 @@ vardomh <- function(Y, H, PSU, w_final,
 
       
   # Ratio of two totals
-  lin_outp <- NULL
-  if (!is.null(Z)) {
-    if (!is.null(Dom1)) Z1 <- domain(Z, Dom1) else Z1 <- Z
-    Y2 <- lin.ratio(Y1, Z1, w_final, Dom=NULL)
-    Y2a <- lin.ratio(Y1, Z1, w_design, Dom=NULL)
-    if (any(is.na(Y2))) print("Results are calculated, but there are cases where Z = 0")
-    if (outp_lin) ratio_outp <- data.table(id, PSU, Y2) 
-  } else {
-           Y2 <- Y1
-           Y2a <- Y1
-         }
-
-  # Calibration
-
-
-  .SD <- NULL
+  
+  Z1 <- persort <- lin_outp <- estim <- NULL
+  var_est2 <- se <- rse <- cv <- NULL
+  absolute_margin_of_error <- NULL
+  relative_margin_of_error <- CI_lower <- NULL
+  CI_upper <- variable <- variableZ <- .SD <- NULL
+  deff_sam <- deff_est <- deff <- NULL
+  
   aH <- names(H)
   idper <- id
   if (!is.null(period)) idper <- data.table(idper, period)
+
+  if (!is.null(Z)) {
+     if (!is.null(Dom)) Z1 <- domain(Z, Dom) else Z1 <- Z
+            
+     if (is.null(period)) {
+          Y2 <- lin.ratio(Y1, Z1, w_final, Dom=NULL)
+          Y2a <- lin.ratio(Y1, Z1, w_design, Dom=NULL)
+        } else {
+          periodap <- do.call("paste", c(as.list(period), sep="_"))
+          sorts <- unlist(split(Y1[, .I], periodap))
+
+          lin1 <- lapply(split(Y1[, .I], periodap), function(i) lin.ratio(Y1[i], Z1[i], w_final[i], Dom=NULL))
+          Y2 <- rbindlist(lin1)[sorts]
+           
+          lin2 <- lapply(split(Y1[, .I], periodap), function(i) lin.ratio(Y1[i], Z1[i], w_design[i], Dom=NULL))
+          Y2a <- rbindlist(lin2)[sorts]
+        }
+     if (any(is.na(Y2))) print("Results are calculated, but there are cases where Z = 0")
+     if (outp_lin) ratio_outp <- data.table(idper, PSU, Y2) 
+    } else {
+            Y2 <- Y1
+            Y2a <- Y1
+          }
+  lin1 <- lin2 <- NULL
+
+  # Calibration
+
   YY <- data.table(idper, ID_household, H, PSU, w_final, Y2)
   YYa <- data.table(idper, ID_household, H, PSU, w_design, Y2a)
-  idper <- NULL
   
   YY2 <- YY[, lapply(.SD, sum, na.rm = T), by = c(names(YY)[c(2:(5+np))]), .SDcols = names(YY)[-(1:(5+np))]]
   YY2a <- YYa[, lapply(.SD, sum, na.rm = T), by = c(names(YYa)[c(2:(5+np))]), .SDcols = names(YYa)[-(1:(5+np))]]
@@ -364,155 +381,182 @@ vardomh <- function(Y, H, PSU, w_final,
   PSU <- YY2[, np+3, with=F]
   setnames(PSU, names(PSU), aPSU)
 
-  w_final2 <- data.frame(YY2)[,np+4]
-  w_design2 <- data.frame(YY2a)[,np+4]    
-  YY <- YYa <- NULL
-  YY2 <- YY2a <- NULL
+  w_final2 <- YY2[[np+4]]
+  w_design2 <- YY2a[[np+4]]    
+  YY <- YYa <- YY2 <- YY2a <- NULL
 
   # Calibration
   res_outp <- NULL
   if (!is.null(X)) {
        if (np>0) IDh <- data.table(period, IDh)
        setnames(IDh, names(IDh), names(X_ID_household))
-       X0 <- data.table(X_ID_household, g, q, X)
-       D1 <- merge(IDh, X0, by=names(IDh))
-       g <- data.frame(D1)[,np+2]
-       q <- data.frame(D1)[,np+3]
-       X <- D1[,(np+4):ncol(D1),with=F]
-            
-       Y4 <- residual_est(Y=Y3, X=X, weight=w_design2,
-                          q=q, period=period, ind_gr=ind_gr)  
+       X0 <- data.table(X_ID_household, ind_gr, q, g, X)
+       D1 <- data.table(merge(IDh, X0, by=names(IDh)))
+
+       ind_gr <- D1[, np+2, with=F]
+       if (!is.null(period)) ind_gr <- data.table(D1[, names(periodX), with=F], ind_gr)
+       ind_period <- do.call("paste", c(as.list(ind_gr), sep="_"))
+       sorts <- unlist(split(Y2[, .I], ind_period))
+    
+       lin1 <- lapply(split(Y2[, .I], ind_period), function(i) 
+                   residual_est(Y=Y3[i],
+                                X=D1[i,(np+5):ncol(D1),with=F],
+                                weight=w_design2[i],
+                                q=D1[i, np+3, with=F]))
+       Y4 <- rbindlist(lin1)[sorts]
        if (outp_res) res_outp <- data.table(IDh, PSU, w_final2, Y4)
    } else Y4 <- Y3
   Y3 <- NULL
-
-  var_est <- variance_est(Y=Y4, H=H, PSU=PSU, w_final=w_final2,
-                          N_h=N_h, fh_zero=fh_zero, PSU_level=PSU_level,
+                                
+  var_est <- variance_est(Y=Y4, H=H, PSU=PSU,
+                          w_final=w_final2, N_h=N_h, 
+                          fh_zero=fh_zero, PSU_level=PSU_level,
                           period=period, dataset=NULL)
-  np <- sum(ncol(period))
-  if (np>0) var_est <- data.table(t(colSums(var_est[,-c(1:np),with=F], na.rm=T)))
-
+  var_est <- transpos(var_est, is.null(period), "var_est", names(period))
+  all_result <- var_est
 
   # Variance of HT estimator under current design
   var_cur_HT <- variance_est(Y=Y3a, H=H, PSU=PSU, w_final=w_design2,
                              N_h=N_h, fh_zero=fh_zero, PSU_level=PSU_level,
                              period=period, dataset=NULL)
-  if (np>0) var_cur_HT <- data.table(t(colSums(var_cur_HT[,-c(1:np),with=F], na.rm=T)))
+  var_cur_HT <- transpos(var_cur_HT, is.null(period), "var_cur_HT", names(period))
+  all_result <- merge(all_result, var_cur_HT)
+  var_est <-  var_cur_HT <- NULL
+  H <- PSU <- N_h <- NULL
 
   # Variance of HT estimator under SRS
-  var_srs_HT <- var_srs(Y=Y3a, w = w_design2)
-  
+  if (is.null(period)) {
+           var_srs_HT <- var_srs(Y3a, w = w_design2)
+       } else {
+           period_agg <- unique(period)
+           lin1 <- lapply(1:nrow(period_agg), function(i) {
+                          per <- period_agg[i,][rep(1, nrow(Y3a)),]
+                          ind <- (rowSums(per == period) == ncol(period))
+                          data.table(period_agg[i,], 
+                                     var_srs(Y3a[ind], w = w_design2[ind]))
+                        })
+           var_srs_HT <- rbindlist(lin1)
+      }
+  var_srs_HT <- transpos(var_srs_HT, is.null(period), "var_srs_HT", names(period))
+  all_result <- merge(all_result, var_srs_HT)
+
   # Variance of calibrated estimator under SRS
-  var_srs_ca <- var_srs(Y=Y4, w = w_final2)
+  if (is.null(period)) {
+           var_srs_ca <- var_srs(Y4, w = w_final2)
+      } else {
+           period_agg <- unique(period)
+           lin1 <- lapply(1:nrow(period_agg), function(i) {
+                          per <- period_agg[i,][rep(1, nrow(Y3a)),]
+                          ind <- (rowSums(per == period) == ncol(period))
+                          data.table(period_agg[i,], 
+                                     var_srs(Y4[ind], w = w_final2[ind]))
+                        })
+           var_srs_ca <- rbindlist(lin1)
+        }
+  var_srs_ca <- transpos(var_srs_ca, is.null(period), "var_srs_ca", names(period))
+  all_result <- merge(all_result, var_srs_ca)
+  var_srs_HT <-  var_srs_ca <- NULL
 
   # Total estimation
-  Hstr <- data.table(rep.int(1,nrow(Y)))
+  Y_nov <- Z_nov <- .SD <- NULL
+
+  hY <- data.table(Y1*w_final)
+  if (is.null(period)) { Y_nov <- hY[, lapply(.SD, sum, na.rm = T), .SDcols = names(Y1)]
+                } else { hY <- data.table(period, hY)
+                         Y_nov <- hY[, lapply(.SD, sum, na.rm = T), keyby=names(period), .SDcols = names(Y1)]
+                       }
+  Y_nov <- transpos(Y_nov, is.null(period), "Y_nov", names(period))
+  all_result <- merge(all_result, Y_nov)
   
-  hY<-data.table(Hstr,Y1*w_final)
-  .SD <- NULL
-  Y_nov <- hY[, lapply(.SD, sum, na.rm = F), by = c(names(hY)[1]),.SDcols = names(hY)[-1]]
+  if (!is.null(Z1)) {
+         YZnames <- data.table(variable=names(Y_nov), variableDZ=names(Z1))
+         setkeyv(YZnames, "variable")
+         setkeyv(all_result, "variable")
+         all_result <- merge(all_result, YZnames)
+         
+         hZ <- data.table(Z1*w_final)
+         if (is.null(period)) { Z_nov <- hZ[, lapply(.SD, sum, na.rm = T), .SDcols = names(Z1)]
+                       } else { hZ <- data.table(period, hZ)
+                                Z_nov <- hZ[, lapply(.SD, sum, na.rm = T), keyby=names(period), .SDcols = names(Z1)]
+                              }
+         Z_nov <- transpos(Z_nov, is.null(period), "Z_nov", names(period), "variableDZ")
+         setkeyv(all_result, "variableDZ")
+         all_result <- merge(all_result, Z_nov)                                            
+      }
 
-  if (!is.null(Z)) { hZ<-data.table(Hstr,Z1*w_final)          
-                     Z_nov <- hZ[, lapply(.SD, sum, na.rm = T), by = c(names(hZ)[1])
-                                 ,.SDcols = names(hZ)[-1]]
-                     estim <- Y_nov[,-1,with=F]/Z_nov[,-1,with=F]
-                   } else estim <- Y_nov[,-1,with=F]
+  vars <- data.table(variable=names(Y1), nr_names=1:ncol(Y1))
+  setkey(vars, "variable")
+  setkey(all_result, "variable")
+  all_result <- merge(vars, all_result)
+                        
+  vars <- idper <- Y1 <- Z1 <- Y_nov <- NULL
+  Z_nov <- hY <- hZ <- YZnames <- dati <- NULL
+ 
+  all_result[, estim:=Y_nov]   
+  all_result[!is.null(Z_nov), estim:=Y_nov/Z_nov]
 
-  estim2 <- data.table(estim)
-  var_est <- data.table(var_est)
-  var_est2 <- var_est
- 
-  if (!is.null(Z)) setnames(estim2,names(estim2),paste0(names(estim2), "_div_", names(Z)))
- 
-  test_v <- (var_est2<0)
-  test_v[is.na(test_v)] <- FALSE
-  if (any(test_v)) stop("Estimation of variance are negative!")
+  if (nrow(all_result[var_est < 0])>0) stop("Estimation of variance are negative!")
  
   # Effect of sample design
-  deff_sam <- var_cur_HT / var_srs_HT
+  all_result[, deff_sam:=var_cur_HT / var_srs_HT]
   
   # Effect of estimator
-  deff_est <- var_est2 / var_cur_HT
+  all_result[, deff_est:= var_est / var_cur_HT]
   
   # Overall effect of sample design and estimator
-  deff <- deff_sam * deff_est
+  all_result[, deff:= deff_sam * deff_est]
+
+  all_result[, var_est2:=var_est]
+  all_result[xor(is.na(var_est2), var_est2 < 0), var_est2:=NA]
+  all_result[, se:=sqrt(var_est2)]
+  all_result[estim!=0, rse:= se/estim]
+  all_result[estim==0, rse:= NA]
+  all_result[, cv:= rse*100]
 
-  var_est3 <- var_est2
-  var_est3[xor(is.na(var_est3), test_v)] <- 0
-  se <- sqrt(var_est3)
-  se[xor(is.na(var_est3), test_v)] <- NA
-  rse <- se/estim2
-  rse[estim2==0] <- NA 
-  cv <- rse*100
   tsad <- qnorm(0.5*(1+confidence))
-  absolute_margin_of_error <- tsad*se 
-  relative_margin_of_error <- tsad*cv
-  CI_lower <- estim2 - tsad*se
-  CI_upper <- estim2 + tsad*se
- 
-  setnames(var_est, names(var_est), paste0(names(estim2), "__var")) 
-  setnames(se, names(se), paste0(names(estim2), "__se"))
-  setnames(rse, names(rse), paste0(names(estim2), "__rse"))
-  setnames(cv, names(cv), paste0(names(estim2), "__cv"))
-  setnames(absolute_margin_of_error, names(absolute_margin_of_error), paste0(colnames(estim2), "__absolute_margin_of_error"))
-  setnames(relative_margin_of_error, names(relative_margin_of_error), paste0(names(estim2), "__relative_margin_of_error"))
-  setnames(CI_lower,names(CI_lower), paste0(names(estim2), "__CI_lower"))
-  setnames(CI_upper,names(CI_upper), paste0(names(estim2), "__CI_upper"))
-  setnames(var_srs_HT, names(var_srs_HT), paste0(names(estim2),"__var_srs_HT"))
-  setnames(var_srs_ca, names(var_srs_ca), paste0(names(estim2),"__var_srs_ca"))
-  setnames(var_cur_HT, names(var_cur_HT), paste0(names(estim2),"__var_cur_HT"))
-  setnames(deff_sam, names(deff_sam), paste0(names(estim2),"__deff_sam"))
-  setnames(deff_est, names(deff_est), paste0(names(estim2),"__deff_est"))
-  setnames(deff, names(deff), paste0(names(estim2),"__deff"))
+  all_result[, absolute_margin_of_error:= tsad*se]
+  all_result[, relative_margin_of_error:= tsad*cv]
+  all_result[, CI_lower:= estim - tsad*se]
+  all_result[, CI_upper:= estim + tsad*se]
 
-  all_result <- cbind(t(estim), t(var_est), t(se), t(rse), t(cv),
-                      t(absolute_margin_of_error),
-                      t(relative_margin_of_error),  
-                      t(CI_lower),t(CI_upper), t(var_srs_HT),
-                      t(var_cur_HT), t(var_srs_ca),
-                      t(deff_sam), t(deff_est), t(deff))
- 
-  colnames(all_result) <- c("estim", "var", "se", "rse", "cv",
-                            "absolute_margin_of_error",
-                            "relative_margin_of_error",
-                            "CI_lower", "CI_upper", "var_srs_HT",
-                            "var_cur_HT", "var_srs_ca", 
-                            "deff_sam", "deff_est", "deff")
+  setnames(all_result, c("variable", "var_est"), c("variableD", "var"))
+  if (!is.null(Z_nov)) { nosrZ <- all_result$variableDZ
+                         nosrZ <- nosrZ[!duplicated(nosrZ)]
+                         nosrZ1 <- data.table(variableZ=t(data.frame(strsplit(nosrZ, "__")))[,c(1)])
+                         nosrZ <- data.table(variableDZ=nosrZ, nosrZ1)
+                         setkeyv(nosrZ, "variableDZ")
+                         setkeyv(all_result, "variableDZ")
+                         all_result <- merge(all_result, nosrZ)
+                         nosrZ <- nosrZ1 <- NULL
+                      }
 
+  nosr <- data.table(variableD=all_result$variableD, t(data.frame(strsplit(all_result$variableD, "__"))))
+  nosr <- nosr[!duplicated(nosr)]
+  nosr <- nosr[, lapply(nosr, as.character)]
+  setnames(nosr, names(nosr)[2], "variable")
 
-  nosr <- data.table(t(data.frame(strsplit(names(estim), "__"))))
-  setnames(nosr,names(nosr)[1],"variable")
-  nosr <- as.matrix(nosr)
-  sakum <- 2  
   if (!is.null(Dom)) {
-       colnames(nosr)[sakum:ncol(nosr)] <- colnames(Dom)
-       nch1 <- matrix(nchar(colnames(Dom))+2, nrow=1)
-       nosr[,sakum:ncol(nosr)] <- substring(nosr[,sakum:ncol(nosr)],nch1[rep(1, nrow(nosr)),],nchar(nosr[,sakum:ncol(nosr)]))  
-     }
+       setnames(nosr, names(nosr)[3:ncol(nosr)], names(Dom))
+       nosr1 <- nosr[, lapply(names(Dom), function(x) {substring(get(x), nchar(x)+2, nchar(get(x)))})] 
+       setnames(nosr1, names(nosr1), names(Dom))
+       setnames(nosr, names(Dom), paste0(names(Dom),"old"))
+       nosr <- data.table(nosr, nosr1)
+    }
+  setkeyv(nosr, "variableD")
+  setkeyv(all_result, "variableD")
+  all_result <- merge(nosr, all_result)
+  nosr <- nosr1 <- NULL
 
-  if (!is.null(Z)) { 
-        Zmat <- data.frame(nosr[,1], names(Z))
-        nosr[,1] <- paste("R", nosr[,1], sep="__", Zmat[,2]) }
+  all_result[!is.null(Z_nov), variable:=paste("R", get("variable"), sep="__", get("variableZ"))] 
 
-  all_result <- data.frame(nosr, all_result)
-  rownames(all_result) <- NULL
-  setnames(estim, names(estim), paste0(names(estim2), "__est")) 
-  list(estim = estim,
-       var = var_est,
-       se = se,
-       rse = rse,
-       cv = cv,
-       absolute_margin_of_error = absolute_margin_of_error,
-       relative_margin_of_error = relative_margin_of_error,
-       CI_lower = CI_lower,
-       CI_upper = CI_upper,
-       var_srs_HT = var_srs_HT,
-       var_cur_HT = var_cur_HT,
-       var_srs_ca = var_srs_ca,
-       deff_sam = deff_sam,
-       deff_est = deff_est,
-       deff = deff,
-       lin_out = lin_outp,
+  variab <- c("estim", "var", "se", "rse", "cv", "absolute_margin_of_error",
+              "relative_margin_of_error", "CI_lower", "CI_upper", "var_srs_HT",
+              "var_cur_HT", "var_srs_ca", "deff_sam", "deff_est", "deff")
+
+  setkeyv(all_result, c("nr_names", names(Dom), names(period)))
+  all_result <- all_result[, c("variable", names(Dom), names(period), variab), with=F]
+  
+  list(lin_out = lin_outp,
        res_out = res_outp,
        all_result = all_result)
 }
