@@ -70,7 +70,7 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, s2g=FALSE, period
        if (any(duplicated(names(period)))) 
                  stop("'period' are duplicate column names: ", 
                       paste(names(period)[duplicated(names(period))], collapse = ","))
-       if (nrow(period) != n) stop("'period' must be the same length as 'inc'")
+       if (nrow(period) != n) stop("'period' must be the same length as 'Y'")
        if(any(is.na(period))) stop("'period' has unknown values")  
   }   
   np <- sum(ncol(period))
@@ -101,8 +101,8 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, s2g=FALSE, period
   # N_h2
   if (!is.null(N_h2)) {
       N_h2 <- data.table(N_h2)
-      if (ncol(N_h2) != np+2) stop(paste0("'N_h' should be ",toString(np+2)," columns"))
-      if (!is.numeric(N_h2[[ncol(N_h2)]])) stop("The last column of 'N_h' should be numerical")
+      if (ncol(N_h2) != np+2) stop(paste0("'N_h2' should be ",toString(np+2)," columns"))
+      if (!is.numeric(N_h2[[ncol(N_h2)]])) stop("The last column of 'N_h2' should be numerical")
       if (any(is.na(N_h2))) stop("'N_h2' has unknown values") 
       if (is.null(names(N_h2))) stop("'N_h2' must be colnames")
 
@@ -164,7 +164,7 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, s2g=FALSE, period
   }
 
   z_h_h2 <- Ys[, lapply(.SD, sum, na.rm = F), keyby = c(names(Ys)[1:(2+np)]),
-                      .SDcols = names(Ys)[-(0:(ncol(Y)+2))]]
+                      .SDcols = names(Ys)[-(0:(ncol(Y)+2+np))]]
 
   setkeyv(z_h_h2, names(z_h_h2)[c(1:(1+np))]) 
 
@@ -182,10 +182,13 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, s2g=FALSE, period
   z_h_h2[, paste0(names(Y),"_sd") := lapply(y_sd, function(x) 
                  (1/n_h1) * x ^ 2  * pop ^ 2 * (1/n_h1 - 1/pop)/(n_h1-1))]
 
-  z_h_h2[n_h1==1,paste0(names(Y),"_sc"):=NA]
-  z_h_h2[n_h1==1,paste0(names(Y),"_sd"):=NA]
+  z_h_h2[n_h1==1, paste0(names(Y),"_sc"):=NA]
+  z_h_h2[n_h1==1, paste0(names(Y),"_sd"):=NA]
 
-  zh2 <- z_h_h2[, lapply(.SD, sum, na.rm = T), keyby = c(names(z_h_h2)[2]),
+  nameszh2 <- names(H2)
+  if (!is.null(period)) nameszh2 <- c(names(period), nameszh2)
+  
+  zh2 <- z_h_h2[, lapply(.SD, sum, na.rm = T), keyby = nameszh2,
                       .SDcols = names(z_h_h2)[-(1:(2+np))]] 
 
   F_h2 <- merge(N_h2, n_h2)
@@ -215,8 +218,13 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, s2g=FALSE, period
   setnames(s2_g, names(s2_g), names(Y))
   if (!s2g) s2_g <- pop2^2*(1/nh2-1/pop2) * s2_g
 
+  if (np>0) s2_g <- data.table(zh2[, names(period), with=F], s2_g)
+
   # Variance_est
-  var_est <- data.table(t(colSums(s2_g, na.rm=T)))
+  if (np==0) {var_est <- data.table(t(colSums(s2_g, na.rm=T)))
+           } else var_est <- s2_g[, lapply(.SD, sum, na.rm=T), 
+                                       keyby = c(names(s2_g)[c(1:np)]),
+                                      .SDcols = names(Y)]
 
   return(var_est)
 }
